<?xml version="1.0" encoding="UTF-8"?>
<?soap-response-to requestType="CustReportFaultReq" namespace="http://www.nrs.eskom.co.za/xmlvend/meter/2.1/schema"?>
<xsl:stylesheet 
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:m0="http://www.nrs.eskom.co.za/xmlvend/meter/2.1/schema"
    xmlns:b0="http://www.nrs.eskom.co.za/xmlvend/base/2.1/schema">
    
    <!-- Import common templates -->
    <xsl:import href="../common/common.xsl"/>
    
    <!-- Match the root element of the request message for MeterFaultReport-->
    <xsl:template priority="1" match="/m0:custReportFaultReq[contains( string(m0:custFaultReport/@xsi:type), 'MeterFaultReport' )]">
        <m0:custReportFaultResp>
            
            <!-- Insert the Common request attributes -->
            <xsl:call-template name="baseResponseAttributes"/>

            <b0:dispHeader>Customer Fault Report</b0:dispHeader>
            <m0:custFaultLog refNo="123456789">
                <xsl:apply-templates select="m0:custDetail"/>
                <m0:faultReport xsi:type="m0:MeterFaultReport">
                    <!--Check if the desc field was populated -->
                    <xsl:if test="./m0:custFaultReport/m0:desc != ''">
                        <m0:desc><xsl:value-of select="./m0:custFaultReport/m0:desc"/></m0:desc>
                    </xsl:if>                    
                    
                    <!-- Populate meter details -->
                    <xsl:apply-templates select="m0:custFaultReport/m0:meterInfo/b0:meterIdentifier"/>  
                    
                </m0:faultReport>
            </m0:custFaultLog>
    
        </m0:custReportFaultResp>
    </xsl:template>
    <!-- Match the root element of the request message for NetworkFaultReport-->
    <xsl:template priority="1" match="/m0:custReportFaultReq[contains( string(m0:custFaultReport/@xsi:type), 'NetworkFaultReport' )]">
        <m0:custReportFaultResp>
            
            <!-- Insert the Common request attributes -->
            <xsl:call-template name="baseResponseAttributes"/>
            
            <b0:dispHeader>Customer Fault Report</b0:dispHeader>
            <m0:custFaultLog refNo="123456789">
                <xsl:apply-templates select="m0:custDetail"/>
                <m0:faultReport xsi:type="m0:NetworkFaultReport">
                    <!--Check if the desc field was populated -->
                    <xsl:if test="./m0:custFaultReport/m0:desc != ''">
                        <m0:desc><xsl:value-of select="./m0:custFaultReport/m0:desc"/></m0:desc>
                    </xsl:if>                    
                 </m0:faultReport>
            </m0:custFaultLog>
            
        </m0:custReportFaultResp>
    </xsl:template>
    
    <!-- Match the root element of the request message for General faults-->
    <xsl:template match="/m0:custReportFaultReq">
        <m0:custReportFaultResp>
            
            <!-- Insert the Common request attributes -->
            <xsl:call-template name="baseResponseAttributes"/>
            
            <b0:dispHeader>Customer Fault Report</b0:dispHeader>
            <m0:custFaultLog refNo="123456789">
                <xsl:apply-templates select="m0:custDetail"/>
                <m0:faultReport>
                    <!--Check if the desc field was populated -->
                    <xsl:if test="./m0:custFaultReport/m0:desc != ''">
                        <m0:desc><xsl:value-of select="./m0:custFaultReport/m0:desc"/></m0:desc>
                    </xsl:if>
                 </m0:faultReport>
            </m0:custFaultLog>
            
        </m0:custReportFaultResp>
    </xsl:template>
    
    <xsl:template match="m0:custDetail">
        <m0:custDetail name="{./@name}" address="{./@address}" />
    </xsl:template>
    
    <!-- Template - Populate Meter Info based on the meter identifier -->
    <xsl:template match="m0:custFaultReport/m0:meterInfo/b0:meterIdentifier">
        <xsl:choose>
            <xsl:when test="contains (string(./@xsi:type), 'MeterNumber' )">
                <m0:meterInfo>
                    <b0:meterIdentifier msno="{./@msno}" xsi:type="b0:MeterNumber"/>
                </m0:meterInfo>  
            </xsl:when>
            <xsl:when test="contains (string(./@xsi:type), 'MeterCard' )">
                <!-- Track2Data with no date field -->
                <xsl:if test="string-length(./b0:track2Data) = '34'">
                    <m0:meterInfo>
                        <b0:meterIdentifier xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,26,6)}" 
                            krn="{substring(./b0:track2Data,34,1)}" 
                            ti="{substring(./b0:track2Data,32,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,24,2)}" tt="{substring(./b0:track2Data,22,2)}"/>
                        </b0:meterIdentifier>
                    </m0:meterInfo>   
                </xsl:if>  
                <!-- Track2Data with date field -->
                <xsl:if test="string-length(./b0:track2Data) = '37'">
                    <m0:meterInfo>
                        <b0:meterIdentifier xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,29,6)}" 
                            krn="{substring(./b0:track2Data,37,1)}" 
                            ti="{substring(./b0:track2Data,35,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,27,2)}" tt="{substring(./b0:track2Data,25,2)}"/>
                        </b0:meterIdentifier>
                    </m0:meterInfo>  
                </xsl:if>    
            </xsl:when>
            <!-- All meter details -->
            <xsl:when test="contains (string(./@xsi:type), 'MeterDetail' )">
                <m0:meterInfo>
                    <b0:meterIdentifier xsi:type="b0:MeterDetail" 
                        msno="{./@msno}"
                        sgc="{./@sgc}"
                        krn="{./@krn}" 
                        ti="{./@ti}">
                        <b0:meterType at="{./b0:meterType/@at}" tt="{./b0:meterType/@tt}"/>
                    </b0:meterIdentifier>
                </m0:meterInfo>                
            </xsl:when>
        </xsl:choose>   
        
    </xsl:template>
  
    
</xsl:stylesheet>
