<?xml version="1.0" encoding="UTF-8"?>
<?soap-response-to requestType="ConfirmCustomerReq" namespace="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"?>
<xsl:stylesheet 
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:r0="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"
    xmlns:b0="http://www.nrs.eskom.co.za/xmlvend/base/2.1/schema">
    
    <!-- Import common templates -->
    <xsl:import href="../common/common.xsl"/>
    
    <!-- Match the root element of the request message -->
    <xsl:template match="/r0:confirmCustomerReq">
        
                <r0:confirmCustomerResp>
                    
                    <!-- Insert the Common request attributes -->
                    <xsl:call-template name="baseResponseAttributes"/>
                    
                    <b0:dispHeader>Confirm Customer Details</b0:dispHeader>
                    
                    <!-- Populate customer details -->
                    <xsl:apply-templates select="r0:idMethod/b0:custIdentifier"/>  
                    <!-- Populate meter details -->
                     <xsl:apply-templates select="r0:idMethod/b0:meterIdentifier"/>  
                     
                </r0:confirmCustomerResp>           
            
    </xsl:template>
 
    <!-- Customer Details Template -->
    <xsl:template match="r0:idMethod/b0:custIdentifier">
        <xsl:choose>
            <!-- Cust ID by Name -->
            <xsl:when test="contains (string(./@xsi:type), 'CustName' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="{./b0:name} Bar" address="10 Foo Lane, Johannesburg, 1700" accNo="12345678901" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="100" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="04085409916"
                        sgc="100600" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>   
                </r0:confirmCustResult>
                <r0:confirmCustResult>
                    <r0:custVendDetail name="{./b0:name} Foo" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="41" />
                    <r0:custVendConfig>
                        <r0:canVend>false</r0:canVend>
                        <r0:fbeDue>false</r0:fbeDue>
                        <r0:maxVendAmt  value="100" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="06686069342"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
            </xsl:when>
            <!-- Cust ID by Address -->
            <xsl:when test="contains (string(./@xsi:type), 'CustAddress' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Foo Bar" address="{./b0:address}" accNo="12345678901" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="500" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="04085409916"
                        sgc="100600" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Bar Foo" address="{./b0:address}" accNo="0987654321" daysLastPurchase="41" />
                    <r0:custVendConfig>
                        <r0:canVend>false</r0:canVend>
                        <r0:fbeDue>false</r0:fbeDue>
                        <r0:maxVendAmt  value="300" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="06686069342"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
            </xsl:when>
            <!-- Cust ID by AccNo -->
            <xsl:when test="contains (string(./@xsi:type), 'CustAccountNo' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="{./@accNo}" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="3000" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="06686069342"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
            </xsl:when>
            <!-- Cust ID by IDNo -->
            <xsl:when test="contains (string(./@xsi:type), 'CustIDNumber' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="1000" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="06686069342"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
            </xsl:when>
        </xsl:choose>
        
    </xsl:template>
    
    <!-- Template - Populate Meter Details based on the meter identifier -->
     <xsl:template match="r0:idMethod/b0:meterIdentifier">
        <xsl:choose>
            <xsl:when test="contains (string(./@xsi:type), 'MeterNumber' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="100" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{./@msno}"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>
            </xsl:when>
            <xsl:when test="contains (string(./@xsi:type), 'MeterCard' )">
                <!-- Track2Data with no date field -->
                <xsl:if test="string-length(./b0:track2Data) = '34'">
                    <r0:confirmCustResult>
                        <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="5" />
                        <r0:custVendConfig>
                            <r0:canVend>true</r0:canVend>
                            <r0:fbeDue>true</r0:fbeDue>
                            <r0:maxVendAmt  value="100" symbol="ZAR"/>
                            <r0:minVendAmt value="10" symbol="ZAR"/>
                        </r0:custVendConfig>                                    
                        <r0:meterDetail xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,26,6)}" 
                            krn="{substring(./b0:track2Data,34,1)}" 
                            ti="{substring(./b0:track2Data,32,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,24,2)}" tt="{substring(./b0:track2Data,22,2)}"/>
                        </r0:meterDetail>
                    </r0:confirmCustResult>  
                </xsl:if>  
                <!-- Track2Data with date field -->
                <xsl:if test="string-length(./b0:track2Data) = '37'">
                    <r0:confirmCustResult>
                        <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="5" />
                        <r0:custVendConfig>
                            <r0:canVend>true</r0:canVend>
                            <r0:fbeDue>true</r0:fbeDue>
                            <r0:maxVendAmt  value="100" symbol="ZAR"/>
                            <r0:minVendAmt value="10" symbol="ZAR"/>
                        </r0:custVendConfig>                                    
                        <r0:meterDetail xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,29,6)}" 
                            krn="{substring(./b0:track2Data,37,1)}" 
                            ti="{substring(./b0:track2Data,35,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,27,2)}" tt="{substring(./b0:track2Data,25,2)}"/>
                        </r0:meterDetail>
                    </r0:confirmCustResult>       
                </xsl:if>    
            </xsl:when>
            <!-- All meter details -->
            <xsl:when test="contains (string(./@xsi:type), 'MeterDetail' )">
                <r0:confirmCustResult>
                    <r0:custVendDetail name="Foo Bar" address="11 Foo Lane, Johannesburg, 1700" accNo="0987654321" daysLastPurchase="5" />
                    <r0:custVendConfig>
                        <r0:canVend>true</r0:canVend>
                        <r0:fbeDue>true</r0:fbeDue>
                        <r0:maxVendAmt  value="100" symbol="ZAR"/>
                        <r0:minVendAmt value="10" symbol="ZAR"/>
                    </r0:custVendConfig>                               
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{./@msno}"
                        sgc="{./@sgc}"
                        krn="{./@krn}" 
                        ti="{./@ti}">
                        <b0:meterType at="{./b0:meterType/@at}" tt="{./b0:meterType/@tt}"/>
                    </r0:meterDetail>
                </r0:confirmCustResult>    
            </xsl:when>                       
        </xsl:choose>   
        
    </xsl:template>
    
</xsl:stylesheet>

