<?xml version="1.0" encoding="UTF-8"?>
<?soap-response-to requestType="PayAccReq" namespace="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"?>
<xsl:stylesheet 
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:r0="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"
    xmlns:b0="http://www.nrs.eskom.co.za/xmlvend/base/2.1/schema">
    
    <!-- Import common templates -->
    <xsl:import href="../common/common.xsl"/>
    <xsl:import href="../common/baseVend.xsl"/>
    
    <!-- Match the root element of the request message -->
    <xsl:template  priority="1" match="/r0:payAccReq">
        
            <r0:payAccResp>
                
                <!-- Insert the Common request attributes -->
                <xsl:call-template name="baseResponseAttributes"/>
                
                <b0:dispHeader>Account Payment - Tax Invoice</b0:dispHeader>
                <b0:clientStatus>
                    <b0:availCredit  value="2000" symbol="ZAR"/>
                    <b0:batchStatus banking="open" sales="open" shift="open"/>
                </b0:clientStatus>
                <b0:utility name="Eskom" address="Foo Road" taxRef="1234567890"/> 
                <b0:vendor name="Foo Corner Store" address="12 Foo Road, Foo Town, 9999" />
                <b0:custVendDetail name="Mr XMLVend" address="10 XMLVend Avenue, XMLVendVille, 8888" contactNo="(011)6789089" accNo="0987654321" locRef="erf34567" daysLastPurchase="41"/>
          
               <r0:payAccReceipt receiptNo="1234567">
             
                    <r0:transactions>
                        
                        <!-- Caculate the transaction amounts-->
                        <xsl:variable name="txAmt" select="./r0:reqAmt/@value"/>
                        
                    <xsl:choose>
                        <!-- Debt Account Payment -->
                        <xsl:when test="./r0:payAccount[contains (string(@xsi:type), 'DebtRecovery' )]">
                                <r0:tx xsi:type="r0:DebtRecoveryTx">
                                    <r0:amt value="{./r0:reqAmt/@value}" symbol="ZAR"/>
                                    <r0:tax xsi:type="b0:VAT" rate="{$VAT}">
                                        <b0:amt value="{format-number((./r0:reqAmt/@value - (./r0:reqAmt/@value div $VATFact)),'###0.00')}" symbol="ZAR"/>
                                    </r0:tax>    
                                    <r0:accDesc>Conventional debt recovery</r0:accDesc>
                                    
                                    <!-- Populate the Acc No from request or a default value -->
                                    <xsl:choose>
                                        <xsl:when test="contains(string(./r0:payAccount/r0:idMethod/b0:custIdentifier/@xsi:type), 'CustAccountNo')">
                                            <r0:accNo><xsl:value-of select="./r0:payAccount/r0:idMethod/b0:custIdentifier/@accNo"/></r0:accNo>    
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <r0:accNo>12345678</r0:accNo>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                    
                                    <r0:balance value="3800.00" symbol="ZAR"/>
                                </r0:tx>
                        </xsl:when>
                        <!-- Service Charge Account Payment --> 
                        <xsl:when test="./r0:payAccount[contains (string(@xsi:type), 'ServiceChrg')]">
                            <r0:tx xsi:type="r0:ServiceChrgTx">
                                <r0:amt value="{./r0:reqAmt/@value}" symbol="ZAR"/>
                                <r0:tax xsi:type="b0:VAT" rate="{$VAT}">
                                    <b0:amt  value="{format-number(($txAmt - ($txAmt div $VATFact)),'###0.00')}" symbol="ZAR"/>
                                </r0:tax>
                                <r0:accDesc>Service Charge</r0:accDesc>
                                <!-- Populate the Acc No from request or a default value -->
                                <xsl:choose>
                                    <xsl:when test="contains(string(./r0:payAccount/r0:idMethod/b0:custIdentifier/@xsi:type), 'CustAccountNo')">
                                        <r0:accNo><xsl:value-of select="./r0:payAccount/r0:idMethod/b0:custIdentifier/@accNo"/></r0:accNo>    
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <r0:accNo>12345678</r0:accNo>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </r0:tx>
                        </xsl:when>
                        <xsl:otherwise>
                                 <!--Generic Account Payment -->                           
                                <r0:tx xsi:type="r0:PayAccTx">
                                    <r0:amt value="{./r0:reqAmt/@value}" symbol="ZAR"/>
                                    <r0:tax xsi:type="b0:VAT" rate="{$VAT}">
                                        <b0:amt  value="{format-number(($txAmt - ($txAmt div $VATFact)),'###0.00')}" symbol="ZAR"/>
                                    </r0:tax>
                                    <r0:accDesc>Generic Account Payment</r0:accDesc>
                                    <!-- Populate the Acc No from request or a default value -->
                                    <xsl:choose>
                                        <xsl:when test="contains(string(./r0:payAccount/r0:idMethod/b0:custIdentifier/@xsi:type), 'CustAccountNo')">
                                            <r0:accNo><xsl:value-of select="./r0:payAccount/r0:idMethod/b0:custIdentifier/@accNo"/></r0:accNo>    
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <r0:accNo>12345678</r0:accNo>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </r0:tx>
                       </xsl:otherwise>
                    </xsl:choose>
                        <r0:lessRound  value="0" symbol="ZAR"/>
                        <!-- Calculate Change -->
                        <xsl:choose>
                            <xsl:when test="./r0:payType/b0:tenderAmt/@value>./r0:reqAmt/@value">
                                <r0:tenderAmt  value="{./r0:payType/b0:tenderAmt/@value}" symbol="{./r0:payType/b0:tenderAmt/@symbol}"/>
                                <r0:change  value="{./r0:payType/b0:tenderAmt/@value - ./r0:reqAmt/@value}" symbol="{./r0:reqAmt/@symbol}"/>
                            </xsl:when>
                            <xsl:when test="./r0:payType/b0:cheqAmt /@value>./r0:reqAmt/@value">
                                <r0:tenderAmt  value="{./r0:payType/b0:cheqAmt/@value}" symbol="{./r0:payType/b0:cheqAmt/@symbol}"/>
                                <r0:change  value="{./r0:payType/b0:cheqAmt/@value - ./r0:reqAmt/@value}" symbol="{./r0:reqAmt/@symbol}"/>
                            </xsl:when>
                            <xsl:when test="./r0:payType/b0:cardAmt/@value>./r0:reqAmt/@value">
                                <r0:tenderAmt  value="{./r0:payType/b0:cardAmt/@value}" symbol="{./r0:payType/b0:cardAmt/@symbol}"/>
                                <r0:change  value="{./r0:payType/b0:cardAmt/@value - ./r0:reqAmt/@value}" symbol="{./r0:reqAmt/@symbol}"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <r0:tenderAmt  value="{./r0:reqAmt/@value}" symbol="{./r0:reqAmt/@symbol}"/>
                                <r0:change  value="{./r0:reqAmt/@value - ./r0:reqAmt/@value}" symbol="{./r0:reqAmt/@symbol}"/>
                            </xsl:otherwise>
                        </xsl:choose>       
                  </r0:transactions>

                   <!-- Populate Meter Details if provided -->
                   <xsl:apply-templates select="r0:payAccount/r0:idMethod/b0:meterIdentifier"/>
                   
               </r0:payAccReceipt>
            </r0:payAccResp>

    </xsl:template>

    <!-- Template - Populate Meter Details based on the meter identifier -->
    <xsl:template match="r0:payAccount/r0:idMethod/b0:meterIdentifier">
        <xsl:choose>
            <xsl:when test="contains (string(./@xsi:type), 'MeterNumber' )">
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{./@msno}"
                        sgc="100611" 
                        krn="1" 
                        ti="07">
                        <b0:meterType at="07" tt="02"/>
                    </r0:meterDetail>
            </xsl:when>
            <xsl:when test="contains (string(./@xsi:type), 'MeterCard' )">
                <!-- Track2Data with no date field -->
                <xsl:if test="string-length(./b0:track2Data) = '34'">
                        <r0:meterDetail xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,26,6)}" 
                            krn="{substring(./b0:track2Data,34,1)}" 
                            ti="{substring(./b0:track2Data,32,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,24,2)}" tt="{substring(./b0:track2Data,22,2)}"/>
                        </r0:meterDetail>
                </xsl:if>  
                <!-- Track2Data with date field -->
                <xsl:if test="string-length(./b0:track2Data) = '37'">
                        <b0:meterDetail xsi:type="b0:MeterDetail" 
                            msno="{substring(./b0:track2Data,7,11)}"
                            sgc="{substring(./b0:track2Data,29,6)}" 
                            krn="{substring(./b0:track2Data,37,1)}" 
                            ti="{substring(./b0:track2Data,35,2)}">
                            <b0:meterType at="{substring(./b0:track2Data,27,2)}" tt="{substring(./b0:track2Data,25,2)}"/>
                        </b0:meterDetail>
                </xsl:if>    
            </xsl:when>
            <!-- All meter details -->
            <xsl:when test="contains (string(./@xsi:type), 'MeterDetail' )">
                    <r0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{./@msno}"
                        sgc="{./@sgc}"
                        krn="{./@krn}" 
                        ti="{./@ti}">
                        <b0:meterType at="{./b0:meterType/@at}" tt="{./b0:meterType/@tt}"/>
                    </r0:meterDetail>
            </xsl:when>                       
        </xsl:choose>   
        
    </xsl:template>
</xsl:stylesheet>