<?xml version="1.0" encoding="UTF-8"?>
<?soap-response-to requestType="VerifyReq" namespace="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"?>
<xsl:stylesheet 
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:r0="http://www.nrs.eskom.co.za/xmlvend/revenue/2.1/schema"
    xmlns:b0="http://www.nrs.eskom.co.za/xmlvend/base/2.1/schema">
    
    <!-- Import common templates -->
    <xsl:import href="../common/common.xsl"/>
    
    <!-- Match the root element of the request message -->
    <xsl:template match="/r0:verifyReq">
        
                    <r0:verifyResp>
                        
                        <!-- Insert the Common request attributes -->
                        <xsl:call-template name="baseResponseAttributes"/>
                        
                        <b0:dispHeader>Verify Token</b0:dispHeader>
                        <b0:clientStatus>
                            <b0:availCredit value="2000" symbol="ZAR"/>
                            <b0:batchStatus banking="open" sales="open" shift="open"/>
                        </b0:clientStatus>
                        <b0:utility name="Eskom" address="Foo Road" taxRef="1234567890"/>
                        <b0:vendor name="Foo Corner Store" address="12 Foo Road, Foo Town, 9999"/>
                        <b0:custVendDetail name="Mr XMLVend" address="10 XMLVend Avenue, XMLVendVille, 8888"
                            contactNo="(011)6789089" accNo="0987654321" locRef="erf34567" daysLastPurchase="41"/>
                        <r0:creditVendTx receiptNo="1234567">
                            <r0:amt value="45" symbol="ZAR"/>
                            <r0:tax xsi:type="b0:VAT" rate="14">
                                <b0:amt value="5.53" symbol="ZAR"/>
                            </r0:tax>
                            <r0:creditTokenIssue xsi:type="b0:SaleCredTokenIssue">
                                <b0:desc>Normal Sale</b0:desc>

                                <!-- Populate meter details -->
                                <xsl:apply-templates select="r0:idMethod/b0:meterIdentifier"/>  
                                
                                <b0:token xsi:type="b0:STS1Token">
                                    <b0:stsCipher>56493153725450313471</b0:stsCipher>
                                </b0:token>
                                                                
                                <b0:units value="86.20" siUnit="kWh"/>
                                <b0:resource xsi:type="b0:Electricity"/>
                            </r0:creditTokenIssue>
                        </r0:creditVendTx>
                        <r0:tokenInfo xsi:type="b0:STS1TokenInfo">
                            <b0:tokenID>1698595</b0:tokenID>
                            <b0:transferAmt>862</b0:transferAmt>
                            <b0:tokenClass>
                                <b0:credit>Electricity Token</b0:credit>
                            </b0:tokenClass>
                        </r0:tokenInfo>
                    </r0:verifyResp>
        
    </xsl:template>
    
    <!-- Template - Populate Meter Details based on the meter identifier -->
    <xsl:template match="r0:idMethod/b0:meterIdentifier">
        <xsl:choose>
            <xsl:when test="contains (string(./@xsi:type), 'MeterNumber' )">
                <b0:meterDetail xsi:type="b0:MeterDetail" 
                    msno="{./@msno}"
                    sgc="100611" 
                    krn="1" 
                    ti="07">
                    <b0:meterType at="07" tt="02"/>
                </b0:meterDetail>
            </xsl:when>
            <xsl:when test="contains (string(./@xsi:type), 'MeterCard' )">
                <!-- Track2Data with no date field -->
                <xsl:if test="string-length(./b0:track2Data) = '34'">
                    <b0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{substring(./b0:track2Data,7,11)}"
                        sgc="{substring(./b0:track2Data,26,6)}" 
                        krn="{substring(./b0:track2Data,34,1)}" 
                        ti="{substring(./b0:track2Data,32,2)}">
                        <b0:meterType at="{substring(./b0:track2Data,24,2)}" tt="{substring(./b0:track2Data,22,2)}"/>
                    </b0:meterDetail>
                </xsl:if>  
                <!-- Track2Data with date field -->
                <xsl:if test="string-length(./b0:track2Data) = '37'">
                    <b0:meterDetail xsi:type="b0:MeterDetail" 
                        msno="{substring(./b0:track2Data,7,11)}"
                        sgc="{substring(./b0:track2Data,29,6)}" 
                        krn="{substring(./b0:track2Data,37,1)}" 
                        ti="{substring(./b0:track2Data,35,2)}">
                        <b0:meterType at="{substring(./b0:track2Data,27,2)}" tt="{substring(./b0:track2Data,25,2)}"/>
                    </b0:meterDetail>
                </xsl:if>    
            </xsl:when>
            <!-- All meter details -->
            <xsl:when test="contains (string(./@xsi:type), 'MeterDetail' )">
                <b0:meterDetail xsi:type="b0:MeterDetail" 
                    msno="{./@msno}"
                    sgc="{./@sgc}"
                    krn="{./@krn}" 
                    ti="{./@ti}">
                    <b0:meterType at="{./b0:meterType/@at}" tt="{./b0:meterType/@tt}"/>
                </b0:meterDetail>
            </xsl:when>
        </xsl:choose>   
        
    </xsl:template>
    
</xsl:stylesheet>
