package xmlhttp.framework.config;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;

public class FrameworkSetup
{
	/** Returns the user-specific directory (creating it if possible) with the
	 * specified relative path, relative to user's home directory.
	 * 
	 * @param relativeDir
	 * @return
	 * @throws IOException If the directory could does not exist, could not be created, or could not be written to.
	 */
	public static File getUserDirectory(String relativeDir) throws IOException
	{
		File dir = new File( getUserDirectory(), relativeDir );
		if (!dir.exists())
		{
			logger.debug("Creating user directory: " + dir.toString());
			dir.mkdirs();
		}
		
		// Ensure directory is usable
		if (dir.exists() && dir.canWrite())
		{
			return dir;
		}
		else
		{
			String err = "Unable to write to directory: " + dir.toString();
			logger.error( err );
			throw new IOException( err );
		}
	}
	
	/** Returns the user's home diretcory */
	public static File getUserDirectory() throws IOException
	{
		return new File( System.getProperty("user.home") );
	}
	
	private static Logger logger = Logger.getLogger(FrameworkSetup.class);
}
