package xmlhttp.framework.config;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

/** A small utility class to inflate zip files, recreating the required structure */
public class ZipInflator
{
	/** Inflates the given  zip file to the given directory */
	public static void inflateZipFile( File zipFile, File destDir ) throws IOException
	{
		logger.debug("Inflating " + zipFile + " to " + destDir);
		ZipFile zFile = new ZipFile( zipFile );
		Enumeration entries = zFile.entries();
		while (entries.hasMoreElements())
		{
			ZipEntry entry = (ZipEntry)entries.nextElement();
			String entryName = entry.getName(); 
			if (entry.isDirectory())
			{
				// Create directory
				new File( destDir, entryName ).mkdirs();
				logger.debug("Created: " + entryName);
			}
			else
			{
				// Unpack file
				File destFile = new File( destDir, entryName);
				logger.debug("Inflating: " + entryName + " -> " + destFile.toString());
				copy( zFile.getInputStream( entry ), new FileOutputStream( destFile ) );
			}
		}
	}
	
	/** Inflates the zip file from the given input stream to the given directory.
	 * First copies the stream to a temporary file, and thereafter invokes 
	 * 'inflate*(File, File)' as per usual. */
	public static void inflateZipStream( InputStream zipFileStream, File destDir) throws IOException
	{
		File tempDir = new File( System.getProperty("java.io.tmpdir") );
		// Create temporary file
		File tempFile = new File( tempDir, String.valueOf( System.currentTimeMillis() ) + "-temp.zip" );
		tempFile.deleteOnExit();
		// Copy to file
		copy( zipFileStream, new FileOutputStream( tempFile ));
		logger.debug("Created temporary ZIP file: " + tempFile.toString());
		// Inflate file
		inflateZipFile( tempFile, destDir );
		// Delete Temp File
		// (Automatically done on JVM exit - is this a good idea?)
	}
	
	/** Copies from the given input stream to the given output stream. Applies buffering,
	 * and closes streams.
	 * @param from
	 * @param to
	 */
	public static void copy(InputStream from, OutputStream to) throws IOException
	{
		BufferedInputStream bin = new BufferedInputStream( from, 32000);
		BufferedOutputStream bout = new BufferedOutputStream( to, 32000);
		try
		{
			int b = bin.read();
			while (b != -1)
			{
				bout.write( b );
				b = bin.read();
			}
		}
		catch (EOFException ignored){}
		finally
		{
			try { bout.flush(); } catch (Exception e){}
			try { bout.close(); } catch (Exception e){}
			try { bin.close(); } catch (Exception e){}
		}
	}
	
	private static Logger logger = Logger.getLogger( ZipInflator.class );
}
