package xmlhttp.framework.io;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.servlet.ServletInputStream;

/**
 * Servlet input stream the reads from an in-memory source (byte array).
 * This class is usually only used internally in ServletRequestWrapper classes that need
 * to customise the Input stream, such as for XML transformation or HTTP request compression.
 *  
 * FIXME: readLine() is not properly implemented! This may cause problems in obscure circumstances.
 * 
 * @author Dawid Loubser (Solms TCD)
 * @version 0.1 (Sep 22, 2004)
 */
public class InMemoryServletInputStream extends ServletInputStream
{
    // Our underlying stream chained to the byte array (cache)
    private ByteArrayInputStream in;

    
    /** Construct an in-memory input stream with the data to be served.
     * @param requestData The data that will act as the HTTP Request body.
     */
    public InMemoryServletInputStream(byte[] requestData)
    {
        in = new ByteArrayInputStream(requestData);
    }

    
    /** Basic service to read one byte from the stream.
     * The other 'read(...)' methods rely on the superclass implementation, which call this
     * method internally.
     *  
     * @see java.io.InputStream#read()
     */
    public int read() throws IOException
    {
        return in.read();
    }
}