package xmlhttp.framework.messaging;

import javax.servlet.http.HttpServletRequest;

import nu.xom.Document;

import za.co.solms.framework.ooxml.soap.SOAPMessage;

/** An extended SOAP message, specifically for use in a Servlets API environment.
 * It exposes several HTTP related constructs to processing components, which, by
 * casting SOAP messages to this class, assume they are in an HTTP environment.
 * 
 * This is one method of exposing request context information (e.g. request
 * attributes, security information, etc) - a more elegant method would be
 * to perform a conceptual mapping of HTTP information to transport-neutral
 * information in a SOAP "Message Context". Until such a context is implemented
 * though, this message class is the preferred method of accessing said information,
 * such as security (certificates), session information, etc.
 * 
 * NOTE: This message never exposes the "response", which would allow
 * arbitrary communication with the user. It is specifically meant to expose
 * only "read-only" (upstream) information.
 * 
 * @author dawidl
 * @version 0.1 (Jan 6, 2006)
 */

public class HttpServletSOAPMessage extends SOAPMessage
{

	/** Creates a message (represented by the full SOAP envelope XOM tree) which
	 * was received on the given request.
	 * 
	 * @param originalRequest
	 * @param fullMessage
	 */
	public HttpServletSOAPMessage(HttpServletRequest originalRequest, Document fullMessage) throws Exception
	{
		super(fullMessage);
		this.request = originalRequest;
	}
	
	public HttpServletRequest getHttpRequest()
	{
		return request;
	}
	
	private transient HttpServletRequest request;
}
