package za.co.eskom.nrs.xmlvend.messages;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import za.co.eskom.nrs.xmlvend.server.XMLVendConstants;
import nu.xom.Document;
import nu.xom.XPathContext;

public class MessageIDFactory
{
	/** KPS - 13/10/2006 - Get message ID from a generic Request message */
	public static MessageID getMessageIDFromRequestMessage( Document requestMessage ) throws MessageUnidentifiableException
	{
		try
		{
			// Get request message ID date/time
			String dateTime = requestMessage.query("/*/base:msgID/@dateTime", xpContext).get(0).getValue();
			Date date = dateFormat.parse( dateTime );
			
			// Get unique number
			String uniqueNumber = requestMessage.query("/*/base:msgID/@uniqueNumber", xpContext).get(0).getValue();
			long number = Long.parseLong(uniqueNumber, 10);
			
			// Construct
			return new MessageID( date, number );
		}
		catch (Exception e)
		{
			throw new MessageUnidentifiableException("Unable to get message ID", e);
		}
	}

	
	/** Get message ID from a Advice Request message */
	public static MessageID getMessageIDFromAdviceMessage( Document adviceMessage ) throws MessageUnidentifiableException
	{
		try
		{
			// Get message ID date/time
			String dateTime = adviceMessage.query("/*/base:adviceReqMsgID/@dateTime", xpContext).get(0).getValue();
			Date date = dateFormat.parse( dateTime );
			
			// Get unique number
			String uniqueNumber = adviceMessage.query("/*/base:adviceReqMsgID/@uniqueNumber", xpContext).get(0).getValue();
			long number = Long.parseLong(uniqueNumber, 10);
			
			// Construct
			return new MessageID( date, number );
		}
		catch (Exception e)
		{
			throw new MessageUnidentifiableException("Unable to get message ID", e);
		}
	}
	
	/** Get message ID from a standard or fault response */
	public static MessageID getMessageIDFromResponseMessage( Document responseMessage ) throws MessageUnidentifiableException
	{
		try
		{
			// Get message ID date/time
			String dateTime = responseMessage.query("//base:reqMsgID/@dateTime", xpContext).get(0).getValue();
			Date date = dateFormat.parse( dateTime );
			
			// Get unique number
			String uniqueNumber = responseMessage.query("//base:reqMsgID/@uniqueNumber", xpContext).get(0).getValue();
			long number = Long.parseLong(uniqueNumber, 10);
			
			// Construct
			return new MessageID( date, number );
		}
		catch (Exception e)
		{
			throw new MessageUnidentifiableException("Unable to get message ID", e);
		}
	}
	
	private static XPathContext xpContext = new XPathContext("base", XMLVendConstants.XMLVEND_BASE_NAMESPACE);
	private static DateFormat dateFormat = new SimpleDateFormat("yyyyMMDDHHmmss");
}