package za.co.eskom.nrs.xmlvend.server;

import java.util.*;

import org.apache.log4j.Logger;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.XPathContext;
import za.co.solms.framework.impl.ooxml.basic.types.MessageTypeExtractor;
import za.co.solms.framework.impl.ooxml.basic.util.NamespaceMap;
import za.co.solms.framework.ooxml.XMLMessage;
import za.co.solms.framework.ooxml.types.SchemaType;
import za.co.solms.framework.ooxml.types.SchemaTypeRegistry;

/** Assembles advice last-response messages */
public class LastResponseAssembler
{
	/** Inserts the given last response into the given response wrapper message.
	 * TODO: Throw better type of exception if type is unknown */
	public XMLMessage buildLastResponse( XMLMessage responseWrapper, XMLMessage lastResponse) throws NoPlaceHolderException, Exception
	{
		// Get (empty) placeholder node for last response
		Nodes results = responseWrapper.getMessage().query("/*/base:lastResponse",xpContext);
		if (results.size() < 1)
		{
			throw new NoPlaceHolderException();
		}
		
		Element placeholder = (Element)results.get(0);
		
		// Get actual last response 
		Element lastResponseRoot = (Element)lastResponse.getMessage().getRootElement().copy();
		
		//11032007 - KPS : check if lastResponse was a SOAP Fault
		String lastResponseRootValue;
		lastResponseRootValue = lastResponseRoot.getQualifiedName();
		String temp = lastResponseRootValue.toString();
		
		Element xmlvendFault = null;
		if (temp.contains("soap:Fault"))
		{
				//Get the xmlvendFault element out of stored last repsonse SOAP fault message				
				String data = "";
			    String name = "";
			    int elementCount;
			   	     
		        //get the Root element (Fault) name and number of child elements
		    	elementCount = 	lastResponseRoot.getChildCount();
		        logger.debug("Element: " + temp);
		        logger.debug("No. child elements: " + elementCount);
		        
		        //get 'detail' elements and number of child elements
		        Element detail = (Element)lastResponseRoot.getChild(elementCount-1);
		        name = detail.getQualifiedName();
		        elementCount = 	detail.getChildCount();
		        logger.debug("Element: " + name);
		        logger.debug("No. child elements: " + elementCount);
		        
		        //get 'xmlvendFaultResp' element and number of child elements
		        xmlvendFault = (Element)detail.getChild(elementCount-1).copy();
		        name = xmlvendFault.getQualifiedName();
		        elementCount = 	xmlvendFault.getChildCount();
		        logger.debug("Element: " + name);
		        logger.debug("No. child elements: " + elementCount);
		        
		        //(Rename 'xmlvendFaultResp' to 'lastResponse')
				xmlvendFault.setLocalName("lastResponse");
				
				xmlvendFault.setNamespaceURI( XMLVendConstants.XMLVEND_BASE_NAMESPACE );
				
				//KPS - 26/07/2006 - This may not the best way to fix the bug as the base namespace is repeated.
				xmlvendFault.setNamespacePrefix("b0");
		        
				logger.debug("lastResponse (XMLVendFault)namespace prefix: " + xmlvendFault.getNamespacePrefix());
				
				//	replace placeholder with xmlvendFault element 
				responseWrapper.getMessage().getRootElement().replaceChild( placeholder, xmlvendFault );
				
//				// Build map of all namespaces in soap:Fault
				NamespaceMap nsMap = new NamespaceMap( lastResponse.getMessage().getRootElement() );
				Map namespaceMap = new HashMap();
				namespaceMap = nsMap.getAllNamespaces();
				
				//get all namespaces and prefixes
				Iterator it=namespaceMap.values().iterator();
		          while(it.hasNext())
		          {
		        	  String mySpace = (String)it.next();
		        	  //get the prefix
		        	  Set<String> myPrefix = nsMap.getPrefixesForNamespace(mySpace);
		        	  String prefix = "xx";
						if (myPrefix.size() < 1)
						{
							// Insert new
							logger.debug("No prefix can be found for :" + mySpace);
						}
						else
						{
							// Use existing (already declared)
							for (String p : myPrefix)
							{
								prefix = p;
							}
						}
												
		        	  logger.debug("Fault prefix:namespaces: " + prefix +":"+ mySpace);
		        	  //add namespaces to xmlvendFault element
		        	  xmlvendFault.addNamespaceDeclaration(prefix, mySpace);
					  
		          }
								
				//	Insert xsi:type attribute for xmlvendFault element
				String qualifiedTypeName =  "b0:XMLVendFaultResp";
				Attribute typeAttribute = new Attribute("type", qualifiedTypeName);
				typeAttribute.setNamespace("xsi", XMLVendConstants.SCHEMA_INSTANCE_NAMESPACE );
				xmlvendFault.addAttribute( typeAttribute );
			}
			else
			{
				// (Rename any other element to latResponse)
				lastResponseRoot.setLocalName("lastResponse");
		
				lastResponseRoot.setNamespaceURI( XMLVendConstants.XMLVEND_BASE_NAMESPACE );
		
				//TODO KPS - 26/07/2006 - This may not the best way to fix the bug as the base namespace is repeated.
				lastResponseRoot.setNamespacePrefix("b0");
		
				logger.debug("lastResponseRoot namespace prefix: " + lastResponseRoot.getNamespacePrefix());
				
				//replave placeholder with previous response root element
				responseWrapper.getMessage().getRootElement().replaceChild( placeholder, lastResponseRoot );
						
				//	Insert xsi:type attribute for lastresponse root element
				//get Type of Last Response
				SchemaType responseType = lastResponse.getType();
				logger.debug("Last response type: " + responseType);
				if (responseType != null)
				{
					// Find namespace prefix for type (otherwise insert it)
					// Build map of all namespaces in template
					NamespaceMap nsMap = new NamespaceMap( responseWrapper.getMessage().getRootElement() );
					Set<String> prefixes = nsMap.getPrefixesForNamespace( responseType.getName().getNameSpace() );
					
					// If namespace unkown, insert new namespace prefix mapping on root element
					String prefix = "xx";
					if (prefixes.size() < 1)
					{
						// Insert new
						lastResponseRoot.addNamespaceDeclaration(prefix, responseType.getName().getNameSpace());
						logger.debug("Unknown, namespace, adding new namespace and prefix: " + prefix + ":" + responseType.getName().getNameSpace());
					}
					else
					{
						// Use existing (already declared)
						for (String p : prefixes)
						{
							prefix = p;
						}
					}
					
					logger.debug("Prefix used: " + prefix);
					
					// Insert xsi:type attribute
					String qualifiedTypeName = prefix + ":" + responseType.getName().getName();
					Attribute typeAttribute = new Attribute("type", qualifiedTypeName);
					typeAttribute.setNamespace("xsi", XMLVendConstants.SCHEMA_INSTANCE_NAMESPACE );
					lastResponseRoot.addAttribute( typeAttribute );
				}
				else
				{
					throw new Exception("Last response message has no schema type, unable to correctly insert");
				}
		}
		return responseWrapper;
				
	}
	
	private XPathContext xpContext = new XPathContext("base", XMLVendConstants.XMLVEND_BASE_NAMESPACE);
	private Logger logger = Logger.getLogger( LastResponseAssembler.class );
	private MessageTypeExtractor typeExtractor;
	

	/** Exception to indicate that response template does not contain suitable placeholder for last response */
	public static class NoPlaceHolderException extends Exception
	{
		public NoPlaceHolderException()
		{
			super("Response template contains no placeholder (base:lastResponse) to insert last response into");
		}
	}
}