package za.co.eskom.nrs.xmlvend.server.advice;

import za.co.eskom.nrs.xmlvend.messages.MessageUnidentifiableException;
import za.co.solms.framework.ooxml.XMLMessage;
import nu.xom.Document;

/** Responsible for storing (and later providing) the last response message
 * sent to (or intended for) the client.
 * @author dawidl
 * @version 0.1 (Apr 10, 2006)
 * @author subramoneyKP
 * @version 0.2 (Aug 17, 2006)
 * Changed the NoLastResponse Exception to NoResponseException; to handle LastResponse and Reversal advice messages.
 */
public interface LastResponseStore
{

	/** Stores the given response message for later retrieval.
	 * @throws MessageUnidentifiableException If the message could not be uniquely identified.
	 *  */
	public void storeMessage( XMLMessage responseMessage ) throws MessageUnidentifiableException;
	
	/** Retrieves the last response message that matches the message identification 
	 * of the given advice request message.
	 * @throws MessageUnidentifiableException If the advice message does not contain sufficient information to identify the last response
	 * @throws NoResponseException If there is no response that matches the given advice request
	 */
	public XMLMessage getLastResponseMessage( XMLMessage adviceRequestMessage )	throws MessageUnidentifiableException, NoResponseException;
	
	/** Checks if request message ID matches with a stored message ID
	 * 
	 * @throws MessageUnidentifiableException If the advice message does not contain sufficient information to identify the last response
	 * @throws DuplicateMsgIDException If there a the request MsgID and stored response MsgID matches.
	 */
	
	public boolean RequestMsgIDPresent( XMLMessage requestMessage )	throws MessageUnidentifiableException, DuplicateMsgIDException;

}