package za.co.eskom.nrs.xmlvend.server.advice.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import nu.xom.Document;
import za.co.eskom.nrs.xmlvend.messages.ClientID;
import za.co.eskom.nrs.xmlvend.messages.MessageIdentifier;
import za.co.eskom.nrs.xmlvend.messages.MessageIdentifierFactory;
import za.co.eskom.nrs.xmlvend.messages.MessageUnidentifiableException;
import za.co.eskom.nrs.xmlvend.server.advice.DuplicateMsgIDException;
import za.co.eskom.nrs.xmlvend.server.advice.LastResponseStore;
import za.co.eskom.nrs.xmlvend.server.advice.NoResponseException;
import za.co.solms.framework.ooxml.XMLMessage;

/** A simple (in-memory) implementation to store the single last message received
 * from a client, identified by a String ID.
 * 
 * @author dawidl
 * @version 0.1 (Jun 1, 2006)
 */
public class InMemoryLastResponseStore implements LastResponseStore
{
	public InMemoryLastResponseStore()
	{
		logger.debug("Created simple in-memory store for last responses");
	}

	public boolean RequestMsgIDPresent( XMLMessage requestMessage )	throws MessageUnidentifiableException, DuplicateMsgIDException
	{
		// Identify
		logger.debug("Matching Request MsgID and stored Response MsgID");
		MessageIdentifier id = MessageIdentifierFactory.identifyRequestMessage( requestMessage.getMessage() );
		logger.debug("Request message identity: " + id);
		
		// Get stored message for client (if any)
		XMLMessage stored = messageStore.get( id.getClientID() );
		if (stored != null)
		{
			// Derive identifier and see if it matches
			MessageIdentifier storedID = MessageIdentifierFactory.identifyResponseMessage( stored.getMessage() );
			logger.debug("ID of last stored message for client: " + storedID);
			
			// Only return it if it matches requested ID
			if (storedID.equals( id ))
			{
				throw new DuplicateMsgIDException("Duplicate message ID detected." );
			}
			else
			{
				//Msg IDs do not match (not present)
				logger.debug("Request MsgID and stored response MsgID do not match for client: " + id.getClientID());
				return false;
			}
			
		}
		else
		{
			//no previous message for Client
			logger.debug("No previous message stored for client.");
			return false;
		}
	}

	
	public XMLMessage getLastResponseMessage(XMLMessage adviceRequestMessage) 
	throws MessageUnidentifiableException, NoResponseException
	{
		// Identify
		logger.debug("Last response requested, deriving an identifier");
		MessageIdentifier id = MessageIdentifierFactory.identifyAdviceMessage( adviceRequestMessage.getMessage() );
		logger.debug("Advice message identity: " + id);
		
		// Get stored message for client (if any)
		XMLMessage stored = messageStore.get( id.getClientID() );
		if (stored != null)
		{
			// Derive identifier and see if it matches
			MessageIdentifier storedID = MessageIdentifierFactory.identifyResponseMessage( stored.getMessage() );
			logger.debug("ID of last stored message for client: " + storedID);
			
			// Only return it if it matches requested ID
			if (storedID.equals( id ))
			{
				return stored;
			}
			else
			{
				throw new NoResponseException("Stored response does not match requested message.");
			}
			
		}
		else
		{
			throw new NoResponseException("No response stored for client: " + id.getClientID());
		}
	}
	
	
	public void storeMessage(XMLMessage responseMessage) throws MessageUnidentifiableException
	{
		// Identify
		logger.debug("Request to store message, deriving an identifier (for response message)");
		MessageIdentifier id = MessageIdentifierFactory.identifyResponseMessage( responseMessage.getMessage() );
		logger.debug("Message identifier: " + id);
		
		// Store in map
		XMLMessage old = messageStore.put( id.getClientID(), responseMessage );
		if (old == null)
		{
			logger.debug("Store last response for client: " + id.getClientID());
		}
		else
		{
			logger.debug("Replaced previously stored last response for client: " + id.getClientID());
		}
	}
	
	
	private Map<ClientID, XMLMessage> messageStore = new HashMap<ClientID, XMLMessage>();
	private static Logger logger = Logger.getLogger( InMemoryLastResponseStore.class );
}
