package za.co.eskom.nrs.xmlvend.server.security;

import java.security.cert.X509Certificate;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import xmlhttp.framework.messaging.HttpServletSOAPMessage;
import za.co.eskom.nrs.xmlvend.messages.ClientID;
import za.co.eskom.nrs.xmlvend.messages.ClientIDFactory;
import za.co.eskom.nrs.xmlvend.messages.MessageUnidentifiableException;
import za.co.eskom.nrs.xmlvend.server.XMLVendConstants;
import za.co.solms.framework.ooxml.XMLMessage;

import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.XPathContext;
/** This Obtains the CN from the certficate if present, and the ClientID from the request message.
 * It then compares them and returns true if equal or no SSL.
 * Assumtion: The first certificate is the client cetificate.
 * Note, the type of the clientID is not considered for the comparasion ie. the clientID can be an EAN or Generic type.
 * @author subramoneyKP
 * @version 0.1 (December 6, 2006)
 */

public class CommonNameClientIDCheck 
{
	public void cnClientIDcheck (XMLMessage input) throws ClientIDSSLException, MessageUnidentifiableException
	{
		logger.debug("Checking SSL Attributes");
		checkSSLAttributes(input);
		
		logger.debug("Comparing CN and clientID");
		//Get CN from Certificate
		String cn=null;
		cn = getClientCN(input);
		logger.info("Client CN, " + cn);
		
		// Get clientID
		String id=null;
		id = getClientID(input.getMessage());
		logger.info("ClientID, " + id);
		
		//check if SSL/TLS used
		if (cn != null)
		{
		//compare CN and ClientID
			if (cn.compareTo(id)== 0)
			{
				return; //cn matches clientID
			}
			else
			{
				throw new ClientIDSSLException("Cert CN, " + cn + ", does not match ClientID, " + id + ".");	
			}
		}
		else
		{
			return; //no certificates (SSL/TLS)
		}		
	}

	private void checkSSLAttributes() {
		// TODO Auto-generated method stub
		
	}

	private String getClientCN( XMLMessage input ) throws ClientIDSSLException
	{
		String cn = null;
		String otherCN = null;
		
		HttpServletSOAPMessage message = (HttpServletSOAPMessage)input;
		Object o = message.getHttpRequest().getAttribute("javax.servlet.request.X509Certificate");
		if (o != null)
		{
			logger.info("Request contains X509 certificate(s)");
			X509Certificate[] certs = (X509Certificate[])o;
			int count= 0;
			for (X509Certificate cert : certs)
			{
					
				String dn = cert.getSubjectX500Principal().getName();
				logger.debug("DN = " + dn);
				/*
				 * TODO: Find a better way to determine which is the client certificate.
				 * Currently just getting the CN of the first cert in the collection ????
				 * Suggest suppling the ClientID then iterate through all certs in collection and find where CN = ClientID
				 */		

				//get CN for first certificate only (First certificate is assumed to be client certificate)
				if (count == 0)
				{
//					get cert Comman Name value
					cn = getCertCN(dn);
					//check if CN not found	
					if (cn == null)
					{
						//CN not found throw exception
						throw new ClientIDSSLException("No CN found in client certificate");	
					}
					else
					{
					logger.debug("Client CN = " + cn);
					}
				}
				count++;
			}
		}
		else
		{
			logger.info("Request DOES NOT contain X509 certificate(s)");
		}
		
		return cn;
	}
	
	//get CN from cert DN
	private String getCertCN( String dn ) 
	{
		String cn = null; 
		//get components of the cert DN
		String[] tokens = dn.split(",");
		for (int i = 0; i < tokens.length; ++i) 
		{
		       logger.debug("Cert component: "+ tokens[i]);
		       String[] tokens2 = tokens[i].split("=");
				for (int x = 0; x < tokens2.length; ++x)
				{
					logger.debug("Component part:" + tokens2[x]);
					//chek for "CN" part
					if (tokens2[x].contentEquals("CN"))
					{
						logger.debug("Cert CN value:" + tokens2[x+1]);
						cn = tokens2[x+1];
					}
				}		              
		}
		
		return cn;
		     
	}
	
	private String getClientID( Document xmlvendMessage )
	{
		String id = null;
		// Get first clientID node
		Nodes clientID = xmlvendMessage.query("//base:clientID", xpc);
		if (clientID.size() > 0)
		{
			// First occurrence
			Node c = clientID.get(0);
			
			// Get all atributes on ID element
			Nodes n = c.query("@*", xpc);
			if (n.size() > 0)
			{
				//get the clientID node, filter the "ean" and "id" attributes
		
				for (int i = 0; i < n.size(); i++)
				{
					Attribute attribute = (Attribute)n.get(i);
					
					//check for "ean"
					if (attribute.getLocalName().compareTo("ean") == 0)
					{
						id = attribute.getValue();
					}
					
					//check for "id"
					if (attribute.getLocalName().compareTo("id") == 0)
					{
						id = attribute.getValue();
						logger.debug("clientID = " + id);
					}					
				}
			}
		}
	  return id;
	}
	

	private void checkSSLAttributes(XMLMessage input)
	{

		// security checks
		try
		{
			HttpServletSOAPMessage message = (HttpServletSOAPMessage)input;
			String cyphersuite = (String)message.getHttpRequest().getAttribute("javax.servlet.request.cipher_suite");

			if (cyphersuite != null)
			{
				logger.debug("ciphersuite: " + cyphersuite);
			}
				
	
			Object o = message.getHttpRequest().getAttribute("javax.servlet.request.key_size");
			if (o != null)
			{
				Integer size = (Integer)o;
				logger.debug("Key size: "+ size);
			}
			else
			{
				logger.debug("No Key size!!");
			}
			
			
		
			// from 2.2 spec
			X509Certificate[] certs = (X509Certificate[])message.getHttpRequest().getAttribute("javax.servlet.request.X509Certificate"); 
			if (certs != null && certs.length > 0)
			{
				for (int i=0;i<certs.length;i++)
				{
					X509Certificate cert = certs[i];
					logger.debug(cert.getType());
				}
				
				
			}

		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private static XPathContext xpc = new XPathContext("base",XMLVendConstants.XMLVEND_BASE_NAMESPACE);
	private static Logger logger = Logger.getLogger( CommonNameClientIDCheck.class );
}
