package za.co.eskom.nrs.xmlvend.server.tests;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import javax.xml.transform.Templates;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import za.co.eskom.nrs.xmlvend.server.ResponseMap.XMLFilter;
import junit.framework.TestCase;

/** This test tests the format of the response templates (style sheets) by
 * trying to compile them, as the server would. It finds the xslt files by assuming 
 * that the test suite is called (e.g. by ant task) from the root of the
 * project (where build.xml resides).
 * 
 * @author dawidl
 * @version 0.1 (Feb 14, 2006)
 */
public class ResponseMessageFormatTest extends TestCase
{

	public static void main(String[] args)
	{
		junit.textui.TestRunner.run(ResponseMessageFormatTest.class);
	}

	protected void setUp() throws Exception
	{
		// Logging
		BasicConfigurator.configure();
		// Find all files
		scan();
	}
	

	protected void tearDown() throws Exception
	{
		super.tearDown();
	}
	
	/** Main test */
	public void testTemplates() throws Exception
	{
		logger.info("Compiling XSLT templates to check syntax...");
		for (File f : templates)
		{
			String fName = f.getName();
			if (fName.endsWith(".xsl"))
			{
				logger.info("Testing: " + fName);
				Templates t = TransformerFactory.newInstance().newTemplates( new StreamSource( f ) );
				t.newTransformer();
			}
			else
			{
				logger.info("Ignoring: " + fName + " (Not an XSL stylesheet)");
			}
		}
		
	}
	
	/** Scans for response messages */
	public void scan() throws IOException
	{
		File rd = new File( responseDir );
		logger.info("Finding all style sheets in: " + rd.getAbsolutePath());
		addFilesInDir( rd );
	}

	
	
	/** Recursively locates all XML files, and calls for them to be added */
	protected void addFilesInDir( File dir ) throws IOException
	{
		File[] children = dir.listFiles( new XMLFilter() );
		for (File f : children)
		{
			if (f.isDirectory())
			{
				addFilesInDir( f );
			}
			else
			{
				templates.add( f );
			}
		}
	}

	private Set<File> templates = new HashSet<File>();
	private String responseDir = "source/defaults/responses";
	private static Logger logger = Logger.getLogger( ResponseMessageFormatTest.class ); 
}
