package za.co.solms.framework.impl.ooxml.basic.types.tests;

import org.apache.log4j.BasicConfigurator;
import nu.xom.Builder;
import nu.xom.Document;
import za.co.solms.framework.impl.ooxml.basic.types.MessageTypeExtractor;
import za.co.solms.framework.ooxml.QualifiedName;
import za.co.solms.framework.ooxml.types.SchemaType;
import za.co.solms.framework.ooxml.types.SchemaTypeRegistry;
import junit.framework.TestCase;

public class MessageTypeExtractorTest extends TestCase
{
	public static void main(String[] args)
	{
		junit.textui.TestRunner.run(MessageTypeExtractorTest.class);
	}
	
	public MessageTypeExtractorTest() throws Exception
	{
		typeRegistry = SchemaTypeRegistry.Factory.getInstance();
		BasicConfigurator.configure();
	}
	
	 @Override
	protected void setUp() throws Exception
	{
		 // Create a registry
		 typeRegistry = SchemaTypeRegistry.Factory.getInstance();
		 
		 // Add types from the test schema
		 typeRegistry.addTypesFromSchema( getClass().getClassLoader()
				 .getResource(TEST_BASE + "a.xsd") );
		 
		 // Create a type extractor
		 typeBuilder = new MessageTypeExtractor( typeRegistry );
	}
	 
	public void testDeriveType() throws Exception
	{
		// NON-POLYMORPHIC TESTING
		SchemaType aType = typeRegistry.getTypeByName( new QualifiedName("urn:test:ooxml:typeExtractor","A") );
		SchemaType bType = typeRegistry.getTypeByName( new QualifiedName("urn:test:ooxml:typeExtractor","B") );
		
		// Read valid instance document 1 (xmlns="", no xsi:type) [A]
		Document i = getTestFileXML( "1.xml" );
		SchemaType iT = typeBuilder.deriveType( i.getRootElement() );
		System.out.println("1.xml -> " + iT);
		assertTrue( iT == aType );
		
		// Read valid instance document 2 (xmlns="", with xsi:type) [B]
		Document ii = getTestFileXML( "2.xml" );
		SchemaType iiT = typeBuilder.deriveType( ii.getRootElement() );
		System.out.println("2.xml -> " + iiT);
		assertTrue( iiT == bType );
		
		// Read valid instance document 2a (xmlns="", with xsi:type, type name without prefix) [B]
		Document iia = getTestFileXML( "2a.xml" );
		SchemaType iiaT = typeBuilder.deriveType( iia.getRootElement() );
		System.out.println("2a.xml -> " + iiaT);
		assertTrue( iiaT == bType );
		
		// Read valid instance document 3 (prefixed, no xsi:type) [A]
		Document iii = getTestFileXML( "3.xml" );
		SchemaType iiiT = typeBuilder.deriveType( iii.getRootElement() );
		System.out.println("3.xml -> " + iiiT);
		assertTrue( iiiT == aType );
		
		// Read valid instance document 4 (prefixed, with xsi:type) [B]
		Document iv = getTestFileXML( "4.xml" );
		SchemaType ivT = typeBuilder.deriveType( iv.getRootElement() );
		System.out.println("4.xml -> " + ivT);
		assertTrue( ivT == bType );
		
		// POLYMORPHIC / EXTENSIONS TESTING

		// Read valid instance document 5, with unknown xsi:type but valid 
		// root element
		//...
		
		// Test xsi:schemalocation for unknown namespace (import schema)
		//...
		
		// TODO: NEGATIVE TESTING
		//...
		
	}
	
	
	/** Returns the given test file document */
	private Document getTestFileXML( String fileName ) throws Exception
	{
		return b.build( getClass().getClassLoader().getResourceAsStream( TEST_BASE + fileName ) );
	}

	private MessageTypeExtractor typeBuilder;
	private SchemaTypeRegistry typeRegistry;
	// Base URI fir test files
	private static final String TEST_BASE = "za/co/solms/framework/impl/ooxml/basic/types/tests/";
	// XOM Builder
	private Builder b = new Builder();
}
