package za.co.solms.framework.impl.ooxml.basic.util.tests;

/** Tests namespace map. Requires XOM (which is used by the framework) */

import java.io.InputStream;
import nu.xom.Builder;
import nu.xom.Document;
import za.co.solms.framework.impl.ooxml.basic.util.NamespaceMap;
import za.co.solms.framework.impl.ooxml.basic.util.UnresolvableNamespaceException;
import za.co.solms.framework.ooxml.QualifiedName;
import junit.framework.TestCase;

public class NamespaceMapTest extends TestCase
{

	public static void main(String[] args)
	{
		junit.textui.TestRunner.run(NamespaceMapTest.class);
	}

	protected void setUp() throws Exception
	{
		// Get test document, and create map
		Builder builder = new Builder(false);
		InputStream file = getClass().getClassLoader().getResourceAsStream("za/co/solms/framework/impl/ooxml/basic/util/tests/namespace-test.xml");
		Document doc = builder.build( file );
		
		nsMap = new NamespaceMap( doc.getRootElement() );
	}

	protected void tearDown() throws Exception
	{
		nsMap = null;
	}

	/*
	 * Test method for 'za.co.solms.framework.impl.ooxml.basic.util.NamespaceMap.getNamespaceFoPrefix(String)'
	 */
	public void testGetNamespaceForPrefix() throws UnresolvableNamespaceException
	{
		// On root element (various variations ins tyle)
		assertEquals( nsMap.getNamespaceForPrefix("a"),"urn:test:a" );
		assertEquals( nsMap.getNamespaceForPrefix("b"),"urn:test:b" );
		assertEquals( nsMap.getNamespaceForPrefix("foo"),"urn:bar" ); // I just had to!
		assertEquals( nsMap.getNamespaceForPrefix("c"),"urn:test:c" );
		// Lower in document
		assertEquals( nsMap.getNamespaceForPrefix("e"),"http://test/e" );
		// Non-existent
		boolean e = false;
		try
		{
			nsMap.getNamespaceForPrefix("");
		}
		catch (UnresolvableNamespaceException un)
		{
			e = true;
		}
		assertTrue(e);
	}

	/*
	 * Test method for 'za.co.solms.framework.impl.ooxml.basic.util.NamespaceMap.getPrefixesForNamespace(String)'
	 */
	public void testGetPrefixesForNamespace()
	{
		assertTrue( nsMap.getPrefixesForNamespace("urn:bar").contains("foo") );
		assertTrue( nsMap.getPrefixesForNamespace("urn:bar").contains("foo2") );
		assertEquals( nsMap.getPrefixesForNamespace("urn:bar").size(), 2);
		// Non-existent namespace
		assertNotNull( nsMap.getPrefixesForNamespace("urn:koos") );
		assertEquals( nsMap.getPrefixesForNamespace("urn:koos").size(), 0);
	}
	
	public void testGetAllNamespaces()
	{
		// TODO Test needs to be written!
	}
	
	public void testExpandPrefix() throws UnresolvableNamespaceException
	{
		assertEquals( nsMap.expandPrefix("a:foo"),"urn:test:a" );
		assertEquals( nsMap.expandPrefix("a:Bar"),"urn:test:a" );
		assertEquals( nsMap.expandPrefix("<a:fleem>"),"urn:test:a" );
		assertEquals( nsMap.expandPrefix(" < a:gizmo > "),"urn:test:a" );
		assertNull( nsMap.expandPrefix("koos") );
		
		// TODO: Add tests for: expandPrefix(string, string)
		
		// Negative testing
		boolean e = false;
		
		try
		{
			nsMap.expandPrefix("fff:fff");
			e = false;
		} catch (UnresolvableNamespaceException ee)
		{
			e = true;
		}
		assertTrue(e);
		
	}
	
	public void testExpand() throws UnresolvableNamespaceException
	{
		assertEquals( nsMap.expand("c:koos"), new QualifiedName("urn:test:c","koos") );
		assertEquals( nsMap.expand("foo2:huh"), new QualifiedName("urn:bar","huh") );
	}

	private NamespaceMap nsMap;
}