package za.co.solms.framework.ooxml;

import za.co.solms.framework.ooxml.types.SchemaType;

/** This class refers to an an XML Element (such as a legal root element specified
 * by a schema.) It contains a direct link to the Schema Type of the element.
 * 
 * @author dawidl
 * @version 0.1 (Oct 25, 2005)
 */
public class XMLElementReference
{
	/** Create an XML element of the given name and type (both required) */
	public XMLElementReference (QualifiedName name, SchemaType type) throws IllegalArgumentException
	{
		if (name == null)
			throw new IllegalArgumentException("Attempt to create XML Element without name");
		this.name = name;
		if (type == null)
			throw new IllegalArgumentException("Attempt to create XML Element without type");
		this.type = type;
	}

	
	public QualifiedName getName()
	{
		return name;
	}
	
	
	public SchemaType getType()
	{
		return type;
	}
	
	/** Two elements are considered equal if their names are equal (their types are NOT compared) */
	 @Override
	public boolean equals(Object o)
	{
		 // Delegate to QualifiedName
		 try
		 {
			 XMLElementReference other = (XMLElementReference)o;
			 return other.getName().equals( name );
		 }
		 catch (Exception e)
		 {
			 return false;
		 }
	}
	
	// @Component
	private QualifiedName name;
	// @Associate
	private SchemaType type;
}