package za.co.solms.framework.ooxml;

import java.util.Date;

import za.co.solms.framework.ooxml.types.SchemaType;

import nu.xom.Document;

/** Represents an XML Message, at this stage tied to the XOM object model. 
 * This is a base type (container for a message) which is extended for messages
 * in different domains, e.g. SOAP/HTTP, etc. In addition to the message, the
 * message invocation metadata is also contained.
 * 
 * Such extensions may contain additional methods to access the context or state of 
 * the environment which made the message available, for example, SOAP Headers,
 * HTTP Information, Security, etc.
 * */
public class XMLMessage
{
	public XMLMessage( Document xmlDocument)
	{
		this.document = xmlDocument;
		receiptDate = new Date();
	}
	
	
	/** Returns the actual message, as a XOM document */
	public Document getMessage()
	{
		return this.document;
	}
	
	
	/** Returns the date/time stamp when this message was created. It bears
	 * no relation to the underlying (e.g. file) creation, but simply when the
	 * XML was first received as a message (parsing time) */
	public Date getReceiptDate()
	{
		return receiptDate;
	}
	
	
	/** Returns the TYPE of the message */
	public SchemaType getType()
	{
		return type;
	}
	
	
	/** Sets the type of the message. This method should not be used by processors, 
	 * and will be factored out (using interfaces) in a future version.
	 */
	public void setType( SchemaType type )
	{
		this.type = type;
	}
	
	
	@Override
	public String toString()
	{
		String s = "XML Message";
		if (document != null)
		{
			s += " (" + document.getValue().getBytes().length + " bytes)";
		}
		else
		{
			s += " (empty)";
		}
		return s;
	}
	
	protected Document document;
	private Date receiptDate;
	private SchemaType type;
}