package za.co.solms.framework.ooxml.processing;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/** An annotation to describe the name of the instance document, to be provided as input for a method
 * that processes an XML Document (such as a SOAP message). The method in question
 * has to accept, as single argument (type za.co.solms.framework.ooxml.XMLMessage). 
 * Furthermore, it has to return an XMLMessage, which is considered the 'output' 
 * ('response') to the given input.
 * 
 * @author dawidl
 * @version 0.1 (Nov 3, 2005)
 */

/* Applies to methods only */
@Target( ElementType.METHOD )

/* Must be retained through to runtime */
@Retention( RetentionPolicy.RUNTIME )

/* Methods that use this annotation should be documented as such */
@Documented

public @interface XMLType
{

	/** The name of the type of instance, e.g. "Person" */
	String name() default "";
	
	/** The name space URI of the element, e.g. "http://example.org/foo/bar" */
	String nameSpace() default "";
	
}
