package za.co.solms.framework.ooxml.soap;

import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.XPathContext;
import za.co.solms.framework.ooxml.XMLMessage;

/** A SOAP message, which inherently contains the full SOAP Evelope, but
 * provides services to access the content (payload) and fault, etc.
 * 
 * This class does NOT model the full SOAP/XML Information set (it's not
 * really a data bean) it's more of a read-only transport for SOAP messages.
 * 
 * @author dawidl
 * @version 0.1 (Nov 30, 2005)
 */
public class SOAPMessage extends XMLMessage
{
	/** Construct a SOAP message from a XOM document. The document contains
	 * the full envelope, headers, etc as originall transmitted.
	 * @param xmlDocument
	 */
	public SOAPMessage(Document xmlDocument) throws Exception
	{
		super(xmlDocument);
		
		// Extract various SOAP parts (for now, only body)
		Element body = (Element)xmlDocument.query("/soap:Envelope/soap:Body/*",xpContext).get(0);
		if (body != null)
		{
			// TODO Copying is performance inefficient, but necessary to avoid
			// multiple-parent problem
			this.soapBody = new Document( (Element)body.copy() );
		}
		
		// TODO Add headers, fault, etc.
		//...
	}

	/** Returns the contents of the SOAP Message (the SOAP Body) */
	@Override
	public Document getMessage()
	{
		return soapBody;
	}
	
	/** Returns the complete SOAP message */
	public Document getSOAPEnvelope()
	{
		return super.getMessage();
	}
	
	/** Indicates whether this is a FAULT message or not */
	public boolean isFault()
	{
		if (soapBody == null || soapBody.getRootElement() == null)
		{
			return false;
		}
		else
		{
			Element root = soapBody.getRootElement();
			return root.getLocalName().equals("Fault") 
				&& ( root.getNamespaceURI().equals("") ||
						root.getNamespaceURI().equals( SOAPMessageFactory.SOAP_NAMESPACE ) );
		}
	}
	
	/** If this message is a fault, this produces the fault detail element (if present) */
	public Element getFaultDetail()
	{
		if (!isFault())
		{
			return null;
		}
		else
		{
			// Find first element inside fault detail
			Nodes n = soapBody.query("detail/*",xpContext);
			if (n.size() > 0)
			{
				return (Element)n.get(0);
			}
			else
			{
				return null;
			}
		}
	}
	
	// References just the body bit
	private Document soapBody;
	private static XPathContext xpContext = new XPathContext("soap","http://schemas.xmlsoap.org/soap/envelope/");
}
