package za.co.solms.framework.ooxml.soap;

import nu.xom.Document;
import nu.xom.Element;
import za.co.solms.framework.ooxml.XMLMessage;

/** A singleton factory to create SOAP messages from normal XML or Exceptions.
 * Currently produces SOAP 1.1.
 *  */
public class SOAPMessageFactory
{
	private SOAPMessageFactory()
	{
	}

	/** Obtain a fatcory instance */
	public static SOAPMessageFactory getInstance()
	{
		if (instance == null)
		{
			// TODO: Decouple from implementation
			instance = new SOAPMessageFactory();
		}
		return instance;
	}
	
	
	/** Create a SOAP message containing the specified body content */
	public SOAPMessage createMessage( XMLMessage bodyContent ) throws Exception
	{
		// Create SOAP Envelope
		Element root = new Element("Envelope",SOAP_NAMESPACE);
		Element body = new Element("Body",SOAP_NAMESPACE);
		root.insertChild( body ,0);
		
		// Insert the body contents
		body.insertChild( bodyContent.getMessage().getRootElement().copy(), 0 );
		
		// Wrap in document, and put in SOAP message
		Document msg = new Document( root );
		return new SOAPMessage( msg );
	}
	
	/** Create a SOAP message containing the specified body content */
	public SOAPMessage createMessage( Element bodyContent )
	{
		// Create SOAP Envelope
		Element root = new Element("Envelope",SOAP_NAMESPACE);
		Element body = new Element("Body",SOAP_NAMESPACE);
		root.insertChild( body ,0);
		
		// Insert a copy of the body contents
		body.insertChild( bodyContent.copy(), 0 );
		
		// Wrap in document, and put in SOAP message
		Document msg = new Document( root );
		try
		{
			return new SOAPMessage( msg );
		}
		catch (Exception e)
		{
			// TODO How to handle error here?
			return null;
		}
	}

	private static SOAPMessageFactory instance;
	public static final String SOAP_NAMESPACE = "http://schemas.xmlsoap.org/soap/envelope/";
}
