package za.co.solms.framework.ooxml.types.tests;

import java.util.Set;
import za.co.solms.framework.ooxml.QualifiedName;
import za.co.solms.framework.ooxml.types.SchemaType;
import junit.framework.TestCase;

/** Test cases for the SchemaType class. Since it's a "Value Object", the test cases are pretty boring */

public class SchemaTypeTest extends TestCase
{

	public static void main(String[] args)
	{
		junit.textui.TestRunner.run(SchemaTypeTest.class);
	}

	public SchemaTypeTest(String arg0)
	{
		super(arg0);
	}

	protected void setUp() throws Exception
	{
		super.setUp();
		// Normal types
		full1 = new SchemaType("http://foo.namespace.com/bar/", "Person");
		full1Copy = new SchemaType("http://foo.namespace.com/bar/", "Person");
		full2 = new SchemaType("http://foo.namespace.com/bar/", "Contract");
		// No Namespace
		nons1 = new SchemaType("Student");
		// Extending types
		ext1 = new SchemaType(new QualifiedName("http://foo.namespace.com/bar/", "Employee"), full1);
		ext2 = new SchemaType(new QualifiedName("http://company.com/people/", "Employee"), full1);
	}

	protected void tearDown() throws Exception
	{
		super.tearDown();
	}


	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.getName()'
	 */
	public void testGetName()
	{
		// Positive
		assertEquals( full1.getName(), new QualifiedName("http://foo.namespace.com/bar/", "Person") );
		// Negative
		assertFalse( full1.getName().equals( new QualifiedName("Person", "http://foo.namespace.com/bar/") ) );
		// No namespace
		assertEquals( nons1.getName(), new QualifiedName(null, "Student") );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.getSuperType()'
	 */
	public void testGetSuperType()
	{
		assertEquals( ext1.getSuperType(), full1 );
		assertEquals( ext2.getSuperType(), full1 );
		assertNull( full1.getSuperType() );
	}


	/** This essentially tests polymorphism / abstraction, as the "isOfType()" method enables such behaviour.
	 * Analogous to the 'instanceof' Java keyword */
	public void testIsOfType()
	{
		// Same type
		assertTrue( ext1.isOfType( ext1 ) );
		// Abstract
		assertTrue( ext1.isOfType( full1 ) );
		// Abstract using a copy of abstract type
		assertTrue( ext1.isOfType( full1Copy ) );
		
		// Negative testing
		assertFalse( ext2.isOfType( full2 ) );
		assertFalse( nons1.isOfType( full2 ) );
		assertFalse( ext1.isOfType( null ) );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.getKnownSubTypes()'
	 */
	public void testGetKnownSubTypes()
	{
		Set s = full1.getKnownSubTypes();
		// Check size and contents
		assertTrue( s.size() == 2 );
		assertTrue( s.contains( ext1 ) );
		assertTrue( s.contains( ext2 ) );
		// Shouldn't contain tiself or other types
		assertFalse( s.contains( full1 ) );
		assertFalse( s.contains( nons1 ) );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.getQualifiedName()'
	 */
	public void testGetQualifiedName()
	{
		assertEquals( full1.getName().getQualifiedName(), "{http://foo.namespace.com/bar/}Person" );
		assertEquals( nons1.getName().getQualifiedName(), "Student" );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.toString()'
	 */
	public void testToString()
	{
		// This is not really a test (there is no contract)
		System.out.println("toString() examples follow:");
		System.out.println( full1 );
		System.out.println( full2 );
		System.out.println( ext1 );
		System.out.println( ext2 );
		System.out.println( nons1 );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.equals(Object)'
	 */
	public void testEqualsObject()
	{
		assertTrue( full1.equals( full1Copy ) );
		assertFalse( full1.equals( null ) );
		assertFalse( full1.equals( full2 ) );
		assertFalse( full1.equals( this ) );
	}

	/*
	 * Test method for 'za.co.solms.framework.ooxml.SchemaType.compareTo(Object)'
	 */
	public void testCompareTo()
	{
		// TODO Still to implement...
	}

	// Fixture
	private SchemaType full1, full1Copy, full2, ext1, ext2, nons1;
}